

import reelity.instrument.moon.pocket.vibes.enumerate as enumerate_vibes
	
	
def perform (fields):
	vibes = enumerate_vibes.start ()

	return {
		"status": "pass",
		"note": {
			"vibes": vibes
		}
	}





'''
	import reelity.instrument.moon.connect as moon_connect
	moon_connection = moon_connect.start ()
'''

import pymongo

def start ():
	mongo_client = pymongo.MongoClient ("mongodb://localhost:50003")
	return mongo_client





def present ():
	return """

<!DOCTYPE html>	
<html>
<body
	style="
		padding: 20px;
		background: #222;
		color: #DDD;
		
		font-size: 3em;
	"
>
	<a href="/seed_generator">seed_generator</a>

</body>
</html>

		"""



'''
unicode_string = json.dumps ({
	"move": "send",
	"fields": {
		"to": {
			"kind": "EEC_448_1",
			"address": "3043300506032b6571033a00e26960a83c45c0bb86e356cd727473e96682e76c6dd01c991a6ea0a394ecca27b467554d73e2a22b05425c1926a7a92befda5c1937d6876f00"
		},
		"amount": "40324789324873"
	}
}, indent = 4)		
unsigned_bytes = unicode_string.encode ("utf-8")

import reelity.modules.EEC_448_2.sign as EEC_448_2_sign
signed = EEC_448_2_sign.start (
	private_key_instance,
	unsigned_bytes = unsigned_bytes
)

signed_bytes_hexadecimal = signed ["signed bytes hexadecimal"]
'''

'''
	{
		"name": "performances: perform",
		"fields": {
			"intimate rhythm": {
				"hexadecimal string": ""
			},
			"UTF8 string": ""
		}
	}
'''
'''

'''

import reelity.modules.EEC_448_2.sign as EEC_448_2_sign
import reelity.modules.EEC_448_2.private_key.instance as instantiate_private_key
	
import rich	
	
def story (
	fields
):
	rich.print_json (data = fields)

	private_key_DER_hexadecimal_string = fields ["intimate rhythm"] ["hexadecimal string"]
	UTF8_string = fields ["UTF8 string"]

	unsigned_bytes = UTF8_string.encode ("utf-8")

	private_key_instance = instantiate_private_key.from_DER_hexadecimal_string (
		private_key_DER_hexadecimal_string
	)
	signed = EEC_448_2_sign.start (
		private_key_instance,
		unsigned_bytes = unsigned_bytes
	)

	return {
		"status": "pass",
		"note": {
			"performance": signed ["signed bytes hexadecimal"]
		}
	}





'''
	{
		"name": "hotties: bounce",
		"fields": {
			"flavor": ""
		}
	}
'''



import pymongo

import reelity.instrument.climate as instrument_climate
import reelity.instrument.moon.connect as moon_connect

def perform (fields):
	moon_connection = moon_connect.start ()
	hotties = moon_connection ["pocket"] ["hotties"]
	
	proceeds = collection.delete_one ({
		"flavor": fields ["flavor"]
	})
	if proceeds.deleted_count != 1:
		return {
			"status": "fail",
			"notes": f"{ result.deleted_count } document were deleted."
		}

	return {
		"status": "pass"
	}
	
	
	
	
#




'''
	JavaScript UTF8?
		https://stackoverflow.com/questions/13356493/decode-utf-8-with-javascript
		https://stackoverflow.com/questions/37596748/how-do-i-encode-a-javascript-string-in-utf-16
'''

'''
{
	name: "feelings ECC 448 2: feel",
	fields:  {
		"showy rhythm": {
			"hexadecimal string": showy_rhythm
		},
		"UTF8 story": story,
		"UTF8 performance": performance
	}
}

'''

	
import rich	

import reelity.modules.EEC_448_2.verify as verify

import reelity.modules.EEC_448_2.modulators.byte_string.from_UTF8 as UTF8_to_byte_string
import reelity.modules.EEC_448_2.modulators.byte_string.from_hexadecimal as hexadecimal_to_byte_string

import reelity.modules.EEC_448_2.public_key.instance as instantiate_public_key
	

def performance (
	fields
):
	rich.print_json (data = fields)

	public_key_string = fields ["showy rhythm"] ["hexadecimal string"]
	UTF8_story = fields ["UTF8 story"]
	UTF8_performance = fields ["UTF8 performance"]

	public_key_instance = instantiate_public_key.from_DER_hexadecimal_string (
		public_key_string
	)
	
	print (UTF8_to_byte_string.modulate (UTF8_story))
	print (UTF8_to_byte_string.modulate (UTF8_performance))
	
	#
	#	hexadecimal_to_byte_string.modulate (
	#	UTF8_to_byte_string.module (
	#

	verified = verify.start (
		public_key_instance,
		
		signed_bytes = hexadecimal_to_byte_string.modulate (UTF8_performance),
		unsigned_bytes = UTF8_to_byte_string.modulate (UTF8_story)
	)
	if (verified == "yes"):
		feeling = "euphoria"
	else:
		feeling = "obscure"

	return {
		"status": "pass",
		"note": {
			"feeling": feeling
		}
	}



'''


'''


import os

import reelity.instrument.sockets.clique_group as clique_group
import reelity.instrument.layer._clique as clique_layer

import reelity.stage.moves as stage_moves


def start ():
	import click
	@click.group ("instrument")
	def group ():
		pass
	
	
	'''
		instrument sockets --port 65000
	'''
	import click
	@group.command ("make")
	@click.option ('--layer-port', '-tp', default = '10000')
	@click.option ('--mongo-port', '-mp', default = '10001')
	@click.option ('--name', default = 'instrument-1')
	def search (layer_port, mongo_port, name):	
		CWD = os.getcwd ();
		effect = stage_moves.perform (
			move = {
				"name": "make",
				"fields": {
					"CWD": CWD,
					"name": name,
					"layer port": layer_port,
					"mongo port": mongo_port
				}
			}
		)

		return;
		
		
	
	group.add_command (clique_group.add ())
	group.add_command (clique_layer.add ())
	
	
	
	#group.add_command (instrument_clique_tracks ())
	#group.add_command (instrument_clique_socket ())
	#group.add_command (stage_clique ())
	
	#group ()
	
	return group




#





'''
	import reelity.stage.moves as stage_moves
	effect = stage_moves.perform (
		move = {
			"name": "",
			"fields": {
				
			}
		}
	)
'''

'''
	returns {
		"status": "pass",
		"note": "",
		"fields": {
				
		}
	}
'''

'''
	returns {
		"status": "fail",
		"note": ""
	}
'''


import os
from os.path import dirname, join, normpath

import reelity.stage.moves.names.start_layer as start_layer
import reelity.stage.moves.names.is_on as is_on


#
#	scene index
#
import reelity.stage.moves.names.scene_index.establish as scene_index_establish
import reelity.stage.moves.names.scene_index.freshest as scene_index_freshest

#
#	scene 1
#
import reelity.stage.moves.names.scene_1.produce_Ehh as scene_1_produce_Ehh
import reelity.stage.moves.names.scene_1.add_splendor as scene_1_add_splendor
import reelity.stage.moves.names.scene_1.establish_rules as scene_1_establish_rules

#
#	scene insights
#
import reelity.stage.moves.names.scene_insights.moves.enumerate as scene_insights_moves_enumerate

#
#	scene moves
#
import reelity.stage.moves.names.scene_moves.escrow.send as send_escrow_move
import reelity.stage.moves.names.scene_moves.escrow.receive as receive_escrow_move


moves = {
	"layer: start": start_layer.perform,
	
	#
	#	is on
	#
	"is on": is_on.perform,
	
	
	#
	#	scene index
	#
	"scene index: establish": scene_index_establish.perform,
	"scene index: freshest": scene_index_freshest.perform,
	
	#
	#	scene 1
	#
	"scene 1: produce Ehh": scene_1_produce_Ehh.perform,
	"scene 1: add splendor": scene_1_add_splendor.perform,
	"scene 1: establish rules": scene_1_establish_rules.perform,
	
	#
	#	scene insights
	#		# insights
	#		# descriptions
	#		# impressions
	#
	"scene insights: enumerate moves": scene_insights_moves_enumerate.perform,
	
	#
	#	moves
	#
	"move: send escrow": send_escrow_move.perform,
	"move: receive escrow": receive_escrow_move.perform,
	
}

def records (record):
	print (record)

def perform (
	move = "",
	records = records
):
	if ("name" not in move):
		records (f'The "name" of the move was not given.')
		return;
	
	name = move ["name"];
	if (name in moves):
		print (f"move: { name }")
		
		try:
			return moves [ name ] (move ["fields"])
		except Exception as E:
			return {
				"status": "fail",
				"note": f'An exception occurred. { E }'
			}


	return {
		"status": "fail",
		"note": f'A move named "{ name }" was not found.'
	}








'''
	{
		"name": "hotties: add",
		"fields": {
			"name": "",
			"flavor": ""
		}
	}
'''

import pymongo

import reelity.instrument.climate as instrument_climate
import reelity.instrument.moon.connect as moon_connect

def perform (fields):
	moon_connection = moon_connect.start ()
	hotties = moon_connection ["pocket"] ["hotties"]
	
	existing_document = hotties.find_one ({
		"flavor": flavor
	})
	if existing_document:
		return {
			"status": "fail",
			"note": "That flavor of hottie is already added."
		}
		
	vibes.insert_one ({
		"name": fields ["name"],
		"flavor": flavor
	})

	return {
		"status": "pass"
	}
	
	
	
	
#



'''
	{
		"name": "",
		"fields": {
			"name": "",
			"seed": ""
		}
	}
'''

import pymongo

import reelity.instrument.climate as instrument_climate
import reelity.instrument.moon.connect as moon_connect
import reelity.modules.EEC_448_2.keys as EEC_448_2_key_creator	

def perform (fields):
	assert (len (fields ["seed"]) == 114) 
	assert (len (fields ["name"]) >= 1)

	moon_connection = moon_connect.start ()
	vibes = moon_connection ["pocket"] ["vibes"]
	
	keys = EEC_448_2_key_creator.create (
		seed = fields ["seed"]
	)
	address = "ECC_448_2:" + keys ["public"] ["hexadecimal string"]


	'''
		Check if that vibe already exists.
	'''	
	existing_document = vibes.find_one ({
		"flavor": address
	})
	if existing_document:
		return {
			"status": "fail",
			"note": "A vibe with that address already exists"
		}
	
	vibes.insert_one ({
		"name": fields ["name"],
		"kind": "ECC_448_2",
		"flavor": "ECC_448_2:" + keys ["public"] ["hexadecimal string"],
		"showy": {
			"hexadecimal string": keys ["public"] ["hexadecimal string"]
		},
		"intimate": {
			"hexadecimal string": keys ["private"] ["hexadecimal string"]
		},
		"seed": {
			"hexadecimal string": fields ["seed"]
		}
	})

	return {
		"status": "pass"
	}





'''
	import reelity.modules.EEC_448_2.private_key.scan as private_key_scan
	private_key_scan = private_key_scan.start (path) ["hexadecimal string"]
'''

from Crypto.PublicKey import ECC
from fractions import Fraction

import json

def start (path, format):

	if (format == "JSON"):
		with open (path, mode = 'r') as file:
			JSON_string = file.read ()
			
			return json.loads (JSON_string)
		

	if (format == "DER"):
		with open (path, mode = 'rb') as file:
			byte_string = file.read ()
			instance = ECC.import_key (
				byte_string,
				curve_name = "Ed448"
			)
			hexadecimal_string = byte_string.hex ().upper ()

			class private_key:
				def __init__ (this, instance):
					this.instance = instance
				
			return {
				"DER hexadecimal string": hexadecimal_string,
				"DER byte string": byte_string,
				"instance": instance,
			}
			
	raise Exception ("...")





'''
	
'''

'''
	create_instrument
	
	fields {
		"label": ""
	}
'''



import reelity.instrument.climate as instrument_climate

import os
from os.path import dirname, join, normpath
import pathlib
import sys

def play (fields):

	'''
	import reelity.instrument.climate as instrument_climate
	instrument_climate.build (CWD)

	import reelity.instrument.moon as mongo_node
	mongo_node.start (
		param = {
			"directory": "",
			"port": "27107"
		}
	)
	'''

	offline_climate = instrument_climate.retrieve ()
	instruments_paths = offline_climate ["paths"] ["instrument"]
	
	if ("label" not in fields):
		return {
			"status": "fail",
			"note": f'Please choose a "label" for the instrument.'
		}
	
	instrument_label = fields ["label"]
	instrument_path = str (normpath (join (instruments_paths, instrument_label)))

	if (os.path.isdir (instrument_path) != True):
		os.mkdir (instrument_path)
		instrument_climate.climate ["elected instrument"] ["path"] = instrument_path
		return {
			"status": "pass",
			"note": "instrument created"
		}
		
	else:
		instrument_climate.climate ["elected instrument"] ["path"] = instrument_path
		return {
			"status": "fail",
			"note": "There is already a directory at that path"
		}







import pymongo

import reelity.stage.climate as instrument_climate
import reelity.stage.moon.connect as moon_connect

'''
	
'''
def perform (fields):
	moon_connection = moon_connect.start ()
	scene_1 = moon_connection ["scenes"] ["scene-1"]

	existing_document = scene_1.find_one ({
		"move": "scene 1: produce Ehh"
	})
	if existing_document:
		return {
			"status": "fail",
			"note": "That move has already happened."
		}

	move_number = str (scene_1.count_documents ({}) + 1)

	scene_1.insert_one ({
		"move": "scene 1: produce Ehh",
		"move #": move_number,
		
		"amount": "1e36",
		"flavor": "ECC_448_2:3043300506032B6571033A00E701F06C74F2990640FBBB062B7E1D0D33C2EA2724F9FDD908CACF0F200270907A8400BD26B287335570F34FCA7B0ECD07B0D9622D5CE82A80"
	})


	return {
		"status": "pass"
	}
	
	
	
	
#






import reelity.instrument.layer.start_dev as flask_start_dev
import reelity.instrument.moves as instrument_moves

import atexit
import os
from os.path import dirname, join, normpath
import pathlib
import sys
		
		
def add ():
	import click
	@click.group ("layer")
	def group ():
		pass

	'''
		instrument layer start --label instrument_1 --layer-port 50000 --mongo-port 50001
	'''
	import os
	import click
	@group.command ("start")
	@click.option ('--name', default = 'instrument')
	@click.option ('--layer-port', '-sp', default = '50000')
	@click.option ('--mongo-port', '-mp', default = '50001')
	def start (name, layer_port, mongo_port):
		def stop ():
			print ("--")
			print ("layer start atexit!");
			print ("--")

		def stop_2 ():
			print ("--")
			print ("layer start atexit 2!");
			print ("--")
		
		atexit.register (stop)
		atexit.register (stop_2)
		
		'''
			This might only work if this is called:
				process.wait () 
		'''
		
	
		CWD = os.getcwd ();
		effect = instrument_moves.perform (
			move = {
				"name": "layer: start",
				"fields": {
					"CWD": CWD,
					"name": name,
					"layer port": layer_port,
					"mongo port": mongo_port
				}
			}
		)
	
		print ("effect:", effect)
	
		return;
	
		'''
		import sys
		from os.path import dirname, join, normpath
		import pathlib
		CWD = os.getcwd ();
		mongo_DB_directory = str (normpath (join (CWD, label, "mongo_DB_directory")))
		instrument_path = str (normpath (join (CWD, label)))
		
		import reelity.instrument.climate as instrument_climate
		instrument_climate.build (
			instrument_path = instrument_path
		)
		
		if (not os.path.exists (mongo_DB_directory)):			
			os.mkdir (mongo_DB_directory) 
			
		if (not os.path.isdir (mongo_DB_directory)):
			print ("There is already something at:", mongo_DB_directory)
			return;
		
		from multiprocessing import Process
		
		import reelity.instrument.moon as instrument_mongo
		mongo = Process (
			target = instrument_mongo.start,
			args = (),
			kwargs = {
				"params": {
					"DB_directory": mongo_DB_directory,
					"port": str (mongo_port)
				}
			}
		)
		mongo.start ()
	

		flask_server = Process (
			target = flask_start_dev.start,
			args = (),
			kwargs = {
				"port": service_port
			}
		)
		flask_server.start ()
		
	
		import time
		while True:
			time.sleep (1000)
		'''
		
		return;


	'''
		instrument layer create_safe --label instrument-1
	'''
	import click
	@group.command ("create_safe")
	@click.option ('--label', required = True)
	@click.option ('--port', '-np', default = '50000')
	def create_safe (label, port):	
		address = f"http://127.0.0.1:{ port }"
	
		import json
		from os.path import dirname, join, normpath
		import os
		import requests
		r = requests.patch (
			address, 
			data = json.dumps ({
				"label": "create safe",
				"fields": {
					"label": label
				}
			})
		)
		print (r.text)
		
		return;
		
	return group




#







'''
	python3 "_status/status.proc.py" "instrument/_status/status_1.py"
'''


import os

from os.path import dirname, join, normpath

import atexit
import pathlib
import shlex
import signal
import sys
import subprocess

this_directory_path = str (pathlib.Path (__file__).parent.resolve ())
instrument_start_path = str (normpath (join (this_directory_path, "../start.proc.py")))

def check_1 ():
	#os.system (f'python3 "{ instrument_start_path }" layer start')

	the_process = None

	def stop ():
		print ("atexit!");
		the_process.kill ()

	'''
		This might only work if this is called:
			process.wait () 
	'''
	print ("registering atexit stop")	
	atexit.register (stop)

	import time
	time.sleep (1)

	#command_string = f'python3 "{ instrument_start_path }" layer start'
	#popen_string = shlex.split (command_string);
	
	popen_string = [
		sys.executable,
		instrument_start_path,
		"layer",
		"start"
	]
	
	print ("popen_string:", popen_string)
	
	#
	#	https://stackoverflow.com/questions/4789837/how-to-terminate-a-python-subprocess-launched-with-shell-true/4791612#4791612
	#
	the_process = subprocess.Popen (
		popen_string,
		#shell = True, 
		cwd = this_directory_path,
		
		preexec_fn = os.setsid
	)
	
	print ("process PID:", the_process.pid)
	
	import time
	time.sleep (1)
	
	os.killpg (os.getpgid (the_process.pid), signal.SIGTERM)
	the_process.kill ()
	print ('killed the process', the_process)
	
	time.sleep (2)
	
	print ()
	print ("got to here.")
	print ()

	return;
	
	
checks = {
	'check 1': check_1
}




'''
	import reelity.modules.EEC_448_2.rhythms as EEC_448_2_rhythm_creator
	rhythms = EEC_448_2_rhythm_creator.create (
		seed = ""
	)
'''

'''
	{
		"showy": {
			"hexadecimal string": public_key_DER_hexadecimal_string,
		},
		"intimate": {
			"hexadecimal string": intimate_rhythm_DER_hexadecimal_string
		}
	}
'''

import reelity.modules.EEC_448_2.intimate_rhythm.creator as EEC_448_2_intimate_rhythm_creator
import reelity.modules.EEC_448_2.public_key.creator as EEC_448_2_public_key_creator

def create (
	seed = ""
):
	#
	#	create intimate rhythm
	#
	intimate_rhythm = EEC_448_2_intimate_rhythm_creator.create (seed)
	intimate_rhythm_instance = intimate_rhythm ["instance"]
	intimate_rhythm_DER_hexadecimal_string = intimate_rhythm ["DER hexadecimal string"]
	
	#
	#	create showy rhythm
	#
	public_key = EEC_448_2_public_key_creator.create (
		intimate_rhythm_instance
	)
	public_key_instance = public_key ["instance"]
	public_key_DER_hexadecimal_string = public_key ["DER hexadecimal string"]
	
	return {
		"showy": {
			"hexadecimal string": public_key_DER_hexadecimal_string,
		},
		"intimate": {
			"hexadecimal string": intimate_rhythm_DER_hexadecimal_string
		}
	}










import pymongo

import reelity.instrument.climate as instrument_climate
import reelity.instrument.moon.connect as moon_connect

def perform (fields):
	moon_connection = moon_connect.start ()
	hotties = moon_connection ["pocket"] ["hotties"]
	
	documents_list = list (hotties.find ({}, { "_id": 0 }))
	print (documents_list)

	return {
		"status": "pass"
	}
	
	
	
	
#







import pymongo

import reelity.stage.climate as instrument_climate
import reelity.stage.moon.connect as moon_connect

'''
	{
		"rules": {
			"change >=": "1/2"
		},
		"scenes": {
			"moves per scene": "100"
		},
		"escrow": {
			"scene limit": 100
		},
		"fees": {
			"price": 1000
		}
	}
'''
def perform (fields):
	moon_connection = moon_connect.start ()
	scene_1 = moon_connection ["scenes"] ["scene-1"]

	move = "scene 1: establish rules"

	existing_document = scene_1.find_one ({
		"move": move
	})
	if existing_document:
		return {
			"status": "fail",
			"note": "That move has already occurred."
		}

	move_number = str (scene_1.count_documents ({}) + 1)

	scene_1.insert_one ({
		"move": move,
		"move #": move_number,
		
		"rules": {
			"rules": {
				"change >=": "1/2"
			},
			"scenes": {
				"moves per scene": "100"
			},
			"escrow": {
				"scene limit": "100"
			},
			"fees": {
				"price": "1000"
			}
		}		
	})


	return {
		"status": "pass"
	}
	
	
	
	
#






#
#
import json
from os.path import exists, dirname, normpath, join
#
#


#
#
from flask import Flask, request, send_file, request, make_response, send_from_directory, Response
#from flask_cors import CORS
#from flask_limiter import Limiter
#from flask_limiter.util import get_remote_address
#from flask_socketio import SocketIO, emit
#



def route (
	app = {},
	vue_dist_inventory = None
):
	assert (type (vue_dist_inventory) == dict), type (vue_dist_inventory)

	@app.route ('/', methods = [ 'GET' ])
	def home ():	
		return Response (
			vue_dist_inventory ["index.html"] ["content"], 
			mimetype = vue_dist_inventory [ "index.html" ] ["mime"],
			headers = {}
		)	
				
	@app.route ('/<path:route_path>', methods = [ 'GET' ])
	def send_report (route_path):		
		print ("route:", route_path)
	
		try:
			if (route_path in vue_dist_inventory):
				print ('route is in inventory')
			
				content = vue_dist_inventory [ route_path ] ["content"]
				mimetype = vue_dist_inventory [ route_path ] ["mime"]
				
				headers = {}
				
				return Response (
					content, 
					mimetype = mimetype,
					headers = headers
				)	
					
			else:
				print ("doesn't exist")

		except Exception as E:
			print ("exception:", E)
			pass
			
		return Response (
			vue_dist_inventory ["index.html"] ["content"], 
			mimetype = vue_dist_inventory [ "index.html" ] ["mime"],
			status = 600
		)